package v2

import (
	"context"
	"encoding/base64"
	"fmt"
	"net/http"
	"time"

	"github.com/jarcoal/httpmock"

	papi "github.com/exoscale/egoscale/v2/internal/public-api"
)

var (
	testSKSClusterAddons                     = []string{"exoscale-cloud-controller"}
	testSKSClusterCNI                        = "calico"
	testSKSClusterCreatedAt, _               = time.Parse(iso8601Format, "2020-11-16T10:41:58Z")
	testSKSClusterDescription                = "Test Cluster description"
	testSKSClusterEndpoint                   = fmt.Sprintf("%s.sks-%s.exo.io", testSKSClusterID, testZone)
	testSKSClusterID                         = "df421958-3679-4e9c-afb9-02fb6f331301"
	testSKSClusterName                       = "test-cluster"
	testSKSClusterServiceLevel               = "pro"
	testSKSClusterState                      = "running"
	testSKSClusterVersion                    = "1.18.6"
	testSKSNodepoolAntiAffinityGroupID       = "a266eadc-1e5c-4b0a-a31c-1325d2060434"
	testSKSNodepoolCreatedAt, _              = time.Parse(iso8601Format, "2020-11-18T07:54:36Z")
	testSKSNodepoolDescription               = "Test Nodepool description"
	testSKSNodepoolDiskSize            int64 = 15
	testSKSNodepoolID                        = "6d1eecee-397c-4e16-b103-2d1353bf4ecc"
	testSKSNodepoolInstancePoolID            = "f1f67118-43b6-4632-a709-d55fada62f21"
	testSKSNodepoolInstanceTypeID            = "21624abb-764e-4def-81d7-9fc54b5957fb"
	testSKSNodepoolName                      = "test-nodepool"
	testSKSNodepoolSecurityGroupID           = "efb4f4df-87ce-44e9-b5ee-59a9c1628edf"
	testSKSNodepoolSize                int64 = 3
	testSKSNodepoolState                     = "running"
	testSKSNodepoolTemplateID                = "f270d9a2-db64-4e8e-9cd3-5125887e91aa"
	testSKSNodepoolVersion                   = "1.18.6"
)

func (ts *clientTestSuite) TestSKSCluster_RequestKubeconfig() {
	var (
		testRequestUser   = "test-user"
		testRequestGroups = []string{"system:masters"}
		testKubeconfig    = base64.StdEncoding.EncodeToString([]byte("test"))
	)

	cluster := &SKSCluster{
		ID:   testSKSClusterID,
		c:    ts.client,
		zone: testZone,
	}

	httpmock.RegisterResponder("POST", "/sks-cluster-kubeconfig/"+cluster.ID,
		func(_ *http.Request) (*http.Response, error) {
			resp, err := httpmock.NewJsonResponse(http.StatusOK, struct {
				Kubeconfig string `json:"kubeconfig,omitempty"`
			}{
				Kubeconfig: testKubeconfig,
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}
			return resp, nil
		})

	actual, err := cluster.RequestKubeconfig(context.Background(), testRequestUser, testRequestGroups, time.Hour)
	ts.Require().NoError(err)
	ts.Require().Equal(testKubeconfig, actual)
}

func (ts *clientTestSuite) TestSKSCluster_AddNodepool() {
	var (
		testOperationID    = "08302193-c7e3-42a6-9b3d-da0b2a536577"
		testOperationState = "success"
	)

	httpmock.RegisterResponder("POST", "/sks-cluster/"+testSKSClusterID+"/nodepool",
		func(req *http.Request) (*http.Response, error) {
			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.Operation{
				Id:        &testOperationID,
				State:     &testOperationState,
				Reference: &papi.Reference{Id: &testSKSNodepoolID},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}
			return resp, nil
		})

	httpmock.RegisterResponder("GET", "/operation/"+testOperationID,
		func(req *http.Request) (*http.Response, error) {
			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.Operation{
				Id:        &testOperationID,
				State:     &testOperationState,
				Reference: &papi.Reference{Id: &testSKSNodepoolID},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}
			return resp, nil
		})

	httpmock.RegisterResponder("GET", fmt.Sprintf("/sks-cluster/%s/nodepool/%s",
		testSKSClusterID, testSKSNodepoolID),
		func(_ *http.Request) (*http.Response, error) {
			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.SksNodepool{
				AntiAffinityGroups: &[]papi.AntiAffinityGroup{{Id: &testSKSNodepoolAntiAffinityGroupID}},
				CreatedAt:          &testSKSNodepoolCreatedAt,
				Description:        &testSKSNodepoolDescription,
				DiskSize:           &testSKSNodepoolDiskSize,
				Id:                 &testSKSNodepoolID,
				InstancePool:       &papi.InstancePool{Id: &testSKSNodepoolInstancePoolID},
				InstanceType:       &papi.InstanceType{Id: &testSKSNodepoolInstanceTypeID},
				Name:               &testSKSNodepoolName,
				SecurityGroups:     &[]papi.SecurityGroup{{Id: &testSKSNodepoolSecurityGroupID}},
				Size:               &testSKSNodepoolSize,
				State:              &testSKSNodepoolState,
				Template:           &papi.Template{Id: &testSKSNodepoolTemplateID},
				Version:            &testSKSNodepoolVersion,
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}
			return resp, nil
		})

	cluster := &SKSCluster{
		ID: testSKSClusterID,

		c:    ts.client,
		zone: testZone,
	}

	expected := &SKSNodepool{
		ID:                   testSKSNodepoolID,
		Name:                 testSKSNodepoolName,
		Description:          testSKSNodepoolDescription,
		CreatedAt:            testSKSNodepoolCreatedAt,
		InstancePoolID:       testSKSNodepoolInstancePoolID,
		InstanceTypeID:       testSKSNodepoolInstanceTypeID,
		TemplateID:           testSKSNodepoolTemplateID,
		DiskSize:             testSKSNodepoolDiskSize,
		AntiAffinityGroupIDs: []string{testSKSNodepoolAntiAffinityGroupID},
		SecurityGroupIDs:     []string{testSKSNodepoolSecurityGroupID},
		Version:              testSKSNodepoolVersion,
		Size:                 testSKSNodepoolSize,
		State:                testSKSNodepoolState,
	}

	actual, err := cluster.AddNodepool(context.Background(), expected)
	ts.Require().NoError(err)
	ts.Require().Equal(expected, actual)
}

func (ts *clientTestSuite) TestSKSCluster_UpdateNodepool() {
	var (
		testSKSNodepoolNameUpdated           = testSKSNodepoolName + "-updated"
		testSKSNodepoolDescriptionUpdated    = testSKSNodepoolDescription + "-updated"
		testSKSNodepoolInstanceTypeIDUpdated = testSKSNodepoolInstanceTypeID + "-updated"
		testSKSNodepoolDiskSizeUpdated       = testSKSNodepoolDiskSize + 1
		testOperationID                      = "08302193-c7e3-42a6-9b3d-da0b2a536577"
		testOperationState                   = "success"
	)

	cluster := &SKSCluster{
		ID:   testSKSClusterID,
		c:    ts.client,
		zone: testZone,

		Nodepools: []*SKSNodepool{
			{
				ID:             testSKSNodepoolID,
				Name:           testSKSNodepoolName,
				Description:    testSKSNodepoolDescription,
				InstanceTypeID: testSKSNodepoolInstanceTypeID,
				DiskSize:       testSKSNodepoolDiskSize,
			},
		},
	}

	httpmock.RegisterResponder("PUT", fmt.Sprintf("/sks-cluster/%s/nodepool/%s",
		cluster.ID,
		cluster.Nodepools[0].ID),
		func(req *http.Request) (*http.Response, error) {
			var actual papi.UpdateSksNodepoolJSONRequestBody
			ts.unmarshalJSONRequestBody(req, &actual)
			expected := papi.UpdateSksNodepoolJSONRequestBody{
				Name:         &testSKSNodepoolNameUpdated,
				Description:  &testSKSNodepoolDescriptionUpdated,
				InstanceType: &papi.InstanceType{Id: &testSKSNodepoolInstanceTypeIDUpdated},
				DiskSize:     &testSKSNodepoolDiskSizeUpdated,
			}
			ts.Require().Equal(expected, actual)

			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.Operation{
				Id:        &testOperationID,
				State:     &testOperationState,
				Reference: &papi.Reference{Id: &testSKSNodepoolID},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}

			return resp, nil
		})

	httpmock.RegisterResponder("GET", "/operation/"+testOperationID,
		func(req *http.Request) (*http.Response, error) {
			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.Operation{
				Id:        &testOperationID,
				State:     &testOperationState,
				Reference: &papi.Reference{Id: &testSKSNodepoolID},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}
			return resp, nil
		})

	nodepoolUpdated := SKSNodepool{
		ID:             cluster.Nodepools[0].ID,
		Name:           testSKSNodepoolNameUpdated,
		Description:    testSKSNodepoolDescriptionUpdated,
		InstanceTypeID: testSKSNodepoolInstanceTypeIDUpdated,
		DiskSize:       testSKSNodepoolDiskSizeUpdated,
	}
	ts.Require().NoError(cluster.UpdateNodepool(context.Background(), &nodepoolUpdated))
}

func (ts *clientTestSuite) TestSKSCluster_ScaleNodepool() {
	var (
		testOperationID          = "08302193-c7e3-42a6-9b3d-da0b2a536577"
		testOperationState       = "success"
		testScaleSize      int64 = 3
	)

	cluster := &SKSCluster{
		ID:   testSKSClusterID,
		c:    ts.client,
		zone: testZone,

		Nodepools: []*SKSNodepool{{ID: testSKSNodepoolID}},
	}

	httpmock.RegisterResponder("PUT", fmt.Sprintf("/sks-cluster/%s/nodepool/%s:scale",
		cluster.ID,
		cluster.Nodepools[0].ID),
		func(req *http.Request) (*http.Response, error) {
			var actual papi.ScaleSksNodepoolJSONRequestBody
			ts.unmarshalJSONRequestBody(req, &actual)
			expected := papi.ScaleSksNodepoolJSONRequestBody{Size: testScaleSize}
			ts.Require().Equal(expected, actual)

			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.Operation{
				Id:        &testOperationID,
				State:     &testOperationState,
				Reference: &papi.Reference{Id: &testSKSNodepoolID},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}

			return resp, nil
		})

	httpmock.RegisterResponder("GET", "/operation/"+testOperationID,
		func(req *http.Request) (*http.Response, error) {
			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.Operation{
				Id:        &testOperationID,
				State:     &testOperationState,
				Reference: &papi.Reference{Id: &testSKSNodepoolID},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}
			return resp, nil
		})

	ts.Require().NoError(cluster.ScaleNodepool(context.Background(), cluster.Nodepools[0], testScaleSize))
}

func (ts *clientTestSuite) TestSKSCluster_EvictNodepoolMembers() {
	var (
		testOperationID    = "08302193-c7e3-42a6-9b3d-da0b2a536577"
		testOperationState = "success"
		testMemberID       = "8f288e8d-348f-4361-9994-54d9664019c9"
	)

	cluster := &SKSCluster{
		ID:   testSKSClusterID,
		c:    ts.client,
		zone: testZone,

		Nodepools: []*SKSNodepool{{ID: testSKSNodepoolID}},
	}

	httpmock.RegisterResponder("PUT", fmt.Sprintf("/sks-cluster/%s/nodepool/%s:evict",
		cluster.ID,
		cluster.Nodepools[0].ID),
		func(req *http.Request) (*http.Response, error) {
			var actual papi.EvictSksNodepoolMembersJSONRequestBody
			ts.unmarshalJSONRequestBody(req, &actual)
			expected := papi.EvictSksNodepoolMembersJSONRequestBody{Instances: &[]string{testMemberID}}
			ts.Require().Equal(expected, actual)

			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.Operation{
				Id:        &testOperationID,
				State:     &testOperationState,
				Reference: &papi.Reference{Id: &testSKSNodepoolID},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}

			return resp, nil
		})

	httpmock.RegisterResponder("GET", "/operation/"+testOperationID,
		func(req *http.Request) (*http.Response, error) {
			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.Operation{
				Id:        &testOperationID,
				State:     &testOperationState,
				Reference: &papi.Reference{Id: &testSKSNodepoolID},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}
			return resp, nil
		})

	ts.Require().NoError(cluster.EvictNodepoolMembers(
		context.Background(),
		cluster.Nodepools[0],
		[]string{testMemberID}))
}

func (ts *clientTestSuite) TestSKSCluster_DeleteNodepool() {
	var (
		testOperationID    = "08302193-c7e3-42a6-9b3d-da0b2a536577"
		testOperationState = "success"
	)

	httpmock.RegisterResponder("DELETE",
		fmt.Sprintf("=~^/sks-cluster/%s/nodepool/.*", testSKSClusterID),
		func(req *http.Request) (*http.Response, error) {
			ts.Require().Equal(fmt.Sprintf("/sks-cluster/%s/nodepool/%s",
				testSKSClusterID, testSKSNodepoolID), req.URL.String())

			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.Operation{
				Id:        &testOperationID,
				State:     &testOperationState,
				Reference: &papi.Reference{Id: &testSKSClusterID},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}

			return resp, nil
		})

	httpmock.RegisterResponder("GET", "/operation/"+testOperationID,
		func(req *http.Request) (*http.Response, error) {
			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.Operation{
				Id:        &testOperationID,
				State:     &testOperationState,
				Reference: &papi.Reference{Id: &testSKSNodepoolID},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}
			return resp, nil
		})

	cluster := &SKSCluster{
		ID:   testSKSClusterID,
		c:    ts.client,
		zone: testZone,

		Nodepools: []*SKSNodepool{{ID: testSKSNodepoolID}},
	}

	ts.Require().NoError(cluster.DeleteNodepool(context.Background(), cluster.Nodepools[0]))
}

func (ts *clientTestSuite) TestClient_CreateSKSCluster() {
	var (
		testOperationID    = "08302193-c7e3-42a6-9b3d-da0b2a536577"
		testOperationState = "success"
	)

	httpmock.RegisterResponder("POST", "/sks-cluster",
		func(req *http.Request) (*http.Response, error) {
			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.Operation{
				Id:        &testOperationID,
				State:     &testOperationState,
				Reference: &papi.Reference{Id: &testSKSClusterID},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}
			return resp, nil
		})

	httpmock.RegisterResponder("GET", "/operation/"+testOperationID,
		func(req *http.Request) (*http.Response, error) {
			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.Operation{
				Id:        &testOperationID,
				State:     &testOperationState,
				Reference: &papi.Reference{Id: &testSKSClusterID},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}
			return resp, nil
		})

	httpmock.RegisterResponder("GET", fmt.Sprintf("/sks-cluster/%s", testSKSClusterID),
		func(_ *http.Request) (*http.Response, error) {
			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.SksCluster{
				Addons:      &testSKSClusterAddons,
				Cni:         &testSKSClusterCNI,
				CreatedAt:   &testSKSClusterCreatedAt,
				Description: &testSKSClusterDescription,
				Id:          &testSKSClusterID,
				Level:       &testSKSClusterServiceLevel,
				Name:        &testSKSClusterName,
				State:       &testSKSClusterState,
				Version:     &testSKSClusterVersion,
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}
			return resp, nil
		})

	expected := &SKSCluster{
		AddOns:       testSKSClusterAddons,
		CNI:          testSKSClusterCNI,
		CreatedAt:    testSKSClusterCreatedAt,
		Description:  testSKSClusterDescription,
		ID:           testSKSClusterID,
		Name:         testSKSClusterName,
		Nodepools:    []*SKSNodepool{},
		ServiceLevel: testSKSClusterServiceLevel,
		State:        testSKSClusterState,
		Version:      testSKSClusterVersion,

		c:    ts.client,
		zone: testZone,
	}

	actual, err := ts.client.CreateSKSCluster(context.Background(), testZone, &SKSCluster{
		AddOns:       testSKSClusterAddons,
		CNI:          testSKSClusterCNI,
		Description:  testSKSClusterDescription,
		Name:         testSKSClusterName,
		ServiceLevel: testSKSClusterServiceLevel,
		Version:      testSKSClusterVersion,
	})
	ts.Require().NoError(err)
	ts.Require().Equal(expected, actual)
}

func (ts *clientTestSuite) TestClient_ListSKSClusters() {
	httpmock.RegisterResponder("GET", "/sks-cluster",
		func(_ *http.Request) (*http.Response, error) {
			resp, err := httpmock.NewJsonResponse(http.StatusOK, struct {
				SksClusters *[]papi.SksCluster `json:"sks-clusters,omitempty"`
			}{
				SksClusters: &[]papi.SksCluster{{
					Addons:      &testSKSClusterAddons,
					Cni:         &testSKSClusterCNI,
					CreatedAt:   &testSKSClusterCreatedAt,
					Description: &testSKSClusterDescription,
					Endpoint:    &testSKSClusterEndpoint,
					Id:          &testSKSClusterID,
					Level:       &testSKSClusterServiceLevel,
					Name:        &testSKSClusterName,
					Nodepools: &[]papi.SksNodepool{{
						AntiAffinityGroups: &[]papi.AntiAffinityGroup{{Id: &testSKSNodepoolAntiAffinityGroupID}},
						CreatedAt:          &testSKSNodepoolCreatedAt,
						Description:        &testSKSNodepoolDescription,
						DiskSize:           &testSKSNodepoolDiskSize,
						Id:                 &testSKSNodepoolID,
						InstancePool:       &papi.InstancePool{Id: &testSKSNodepoolInstancePoolID},
						InstanceType:       &papi.InstanceType{Id: &testSKSNodepoolInstanceTypeID},
						Name:               &testSKSNodepoolName,
						SecurityGroups:     &[]papi.SecurityGroup{{Id: &testSKSNodepoolSecurityGroupID}},
						Size:               &testSKSNodepoolSize,
						State:              &testSKSNodepoolState,
						Template:           &papi.Template{Id: &testSKSNodepoolTemplateID},
						Version:            &testSKSNodepoolVersion,
					}},
					State:   &testSKSClusterState,
					Version: &testSKSClusterVersion,
				}},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}
			return resp, nil
		})

	expected := []*SKSCluster{{
		CreatedAt:   testSKSClusterCreatedAt,
		Description: testSKSClusterDescription,
		Endpoint:    testSKSClusterEndpoint,
		ID:          testSKSClusterID,
		Name:        testSKSClusterName,
		Nodepools: []*SKSNodepool{{
			CreatedAt:            testSKSNodepoolCreatedAt,
			Description:          testSKSNodepoolDescription,
			DiskSize:             testSKSNodepoolDiskSize,
			ID:                   testSKSNodepoolID,
			InstancePoolID:       testSKSNodepoolInstancePoolID,
			InstanceTypeID:       testSKSNodepoolInstanceTypeID,
			Name:                 testSKSNodepoolName,
			AntiAffinityGroupIDs: []string{testSKSNodepoolAntiAffinityGroupID},
			SecurityGroupIDs:     []string{testSKSNodepoolSecurityGroupID},
			Size:                 testSKSNodepoolSize,
			State:                testSKSClusterState,
			TemplateID:           testSKSNodepoolTemplateID,
			Version:              testSKSNodepoolVersion,
		}},
		Version:      testSKSClusterVersion,
		ServiceLevel: testSKSClusterServiceLevel,
		CNI:          testSKSClusterCNI,
		AddOns:       testSKSClusterAddons,
		State:        testSKSClusterState,

		c:    ts.client,
		zone: testZone,
	}}

	actual, err := ts.client.ListSKSClusters(context.Background(), testZone)
	ts.Require().NoError(err)
	ts.Require().Equal(expected, actual)
}

func (ts *clientTestSuite) TestClient_ListSKSClusterVersions() {
	var (
		testSKSClusterVersions = []string{
			"1.20.0",
			"1.18.6",
		}
		err error
	)

	httpmock.RegisterResponder("GET", "/sks-cluster-version",
		func(_ *http.Request) (*http.Response, error) {
			resp, err := httpmock.NewJsonResponse(http.StatusOK, struct {
				SksClusterVersions *[]string `json:"sks-cluster-versions,omitempty"`
			}{
				SksClusterVersions: &testSKSClusterVersions,
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}
			return resp, nil
		})

	expected := testSKSClusterVersions
	actual, err := ts.client.ListSKSClusterVersions(context.Background())
	ts.Require().NoError(err)
	ts.Require().Equal(expected, actual)
}

func (ts *clientTestSuite) TestClient_UpdateSKSCluster() {
	var (
		testSKSClusterNameUpdated        = testSKSClusterName + "-updated"
		testSKSClusterDescriptionUpdated = testSKSClusterDescription + "-updated"
		testOperationID                  = "08302193-c7e3-42a6-9b3d-da0b2a536577"
		testOperationState               = "success"
	)

	httpmock.RegisterResponder("PUT", fmt.Sprintf("/sks-cluster/%s", testSKSClusterID),
		func(req *http.Request) (*http.Response, error) {
			var actual papi.UpdateSksClusterJSONRequestBody
			ts.unmarshalJSONRequestBody(req, &actual)
			expected := papi.UpdateSksClusterJSONRequestBody{
				Name:        &testSKSClusterNameUpdated,
				Description: &testSKSClusterDescriptionUpdated,
			}
			ts.Require().Equal(expected, actual)

			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.Operation{
				Id:        &testOperationID,
				State:     &testOperationState,
				Reference: &papi.Reference{Id: &testSKSNodepoolID},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}

			return resp, nil
		})

	httpmock.RegisterResponder("GET", "/operation/"+testOperationID,
		func(req *http.Request) (*http.Response, error) {
			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.Operation{
				Id:        &testOperationID,
				State:     &testOperationState,
				Reference: &papi.Reference{Id: &testSKSNodepoolID},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}
			return resp, nil
		})

	clusterUpdated := SKSCluster{
		ID:          testSKSClusterID,
		Name:        testSKSClusterNameUpdated,
		Description: testSKSClusterDescriptionUpdated,
	}
	ts.Require().NoError(ts.client.UpdateSKSCluster(context.Background(), testZone, &clusterUpdated))
}

func (ts *clientTestSuite) TestClient_UgradeSKSCluster() {
	var (
		testOperationID    = "08302193-c7e3-42a6-9b3d-da0b2a536577"
		testOperationState = "success"
	)

	httpmock.RegisterResponder("PUT",
		fmt.Sprintf("/sks-cluster/%s/upgrade", testSKSClusterID),
		func(req *http.Request) (*http.Response, error) {
			var actual papi.UpgradeSksClusterJSONRequestBody
			ts.unmarshalJSONRequestBody(req, &actual)
			expected := papi.UpgradeSksClusterJSONRequestBody{Version: testSKSClusterVersion}
			ts.Require().Equal(expected, actual)

			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.Operation{
				Id:        &testOperationID,
				State:     &testOperationState,
				Reference: &papi.Reference{Id: &testSKSNodepoolID},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}

			return resp, nil
		})

	httpmock.RegisterResponder("GET", "/operation/"+testOperationID,
		func(req *http.Request) (*http.Response, error) {
			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.Operation{
				Id:        &testOperationID,
				State:     &testOperationState,
				Reference: &papi.Reference{Id: &testSKSNodepoolID},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}
			return resp, nil
		})

	ts.Require().NoError(ts.client.UpgradeSKSCluster(
		context.Background(),
		testZone,
		testSKSClusterID,
		testSKSClusterVersion))
}

func (ts *clientTestSuite) TestClient_DeleteSKSCluster() {
	var (
		testOperationID    = "08302193-c7e3-42a6-9b3d-da0b2a536577"
		testOperationState = "success"
	)

	httpmock.RegisterResponder("DELETE", "=~^/sks-cluster/.*",
		func(req *http.Request) (*http.Response, error) {
			ts.Require().Equal(fmt.Sprintf("/sks-cluster/%s", testSKSClusterID), req.URL.String())

			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.Operation{
				Id:        &testOperationID,
				State:     &testOperationState,
				Reference: &papi.Reference{Id: &testSKSClusterID},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}

			return resp, nil
		})

	httpmock.RegisterResponder("GET", "/operation/"+testOperationID,
		func(req *http.Request) (*http.Response, error) {
			resp, err := httpmock.NewJsonResponse(http.StatusOK, papi.Operation{
				Id:        &testOperationID,
				State:     &testOperationState,
				Reference: &papi.Reference{Id: &testSKSNodepoolID},
			})
			if err != nil {
				ts.T().Fatalf("error initializing mock HTTP responder: %s", err)
			}
			return resp, nil
		})

	ts.Require().NoError(ts.client.DeleteSKSCluster(context.Background(), testZone, testSKSClusterID))
}
